﻿using System;
using System.Collections;
using System.Data;
using System.Data.Sql;
using System.Data.SqlClient;
using System.Drawing;
using System.Diagnostics;
using System.Globalization;
using System.IO;
using System.Threading;
using System.Windows.Forms;
using Excel = Microsoft.Office.Interop.Excel;

namespace CreateExcelReport
{
    class ExcelUILanguageHelper : IDisposable
    {
        private CultureInfo m_CurrentCulture;

        public ExcelUILanguageHelper()
        {
            // save current culture and set culture to en-US
            m_CurrentCulture = Thread.CurrentThread.CurrentCulture;
            Thread.CurrentThread.CurrentCulture = new CultureInfo("en-US");
        }

        // return to normal culture
        public void Dispose() => Thread.CurrentThread.CurrentCulture = m_CurrentCulture;
    }

    class Program
    {
        private const string sqlselectManagers =
            @"select 
                employees.BusinessEntityID, employees.Name, employees.JobTitle, employees.Department, employees.StartDate, 
                managers.name [Reports to], 
                case when employees.JobTitle != managers.JobTitle then managers.JobTitle else '' end [Manager Job Title]
            from 
                (
                    SELECT
                        e.[BusinessEntityID], 
                        p.FirstName + case when p.MiddleName is not null then ' ' + p.MiddleName else '' end + ' ' + p.LastName [Name],
                        e.[JobTitle], d.[Name] AS [Department], edh.[StartDate], 
                        case when e.OrganizationLevel is null then 0 else e.OrganizationLevel end OrganizationLevel,
                        case when e.OrganizationNode is null then '/' else e.OrganizationNode.GetAncestor(1).ToString() end OrganizationNode
                    FROM Person.Person p
                        join HumanResources.Employee e on e.BusinessEntityID = p.BusinessEntityID
                        JOIN [HumanResources].[EmployeeDepartmentHistory] edh ON e.[BusinessEntityID] = edh.[BusinessEntityID] 
                        JOIN [HumanResources].[Department] d ON edh.[DepartmentID] = d.[DepartmentID] 
                    WHERE edh.EndDate IS NULL
                ) employees
                join
                (
                    SELECT
                        e.[BusinessEntityID], 
                        p.FirstName + case when p.MiddleName is not null then ' ' + p.MiddleName else '' end + ' ' + p.LastName [Name],
                        e.[JobTitle], d.[Name] AS [Department], edh.[StartDate], 
                        case when e.OrganizationLevel is null then 0 else e.OrganizationLevel end OrganizationLevel,
                        case when e.OrganizationNode is null then '/' else e.OrganizationNode.ToString() end OrganizationNode
                    FROM Person.Person p
                        join HumanResources.Employee e on e.BusinessEntityID = p.BusinessEntityID
                        JOIN [HumanResources].[EmployeeDepartmentHistory] edh ON e.[BusinessEntityID] = edh.[BusinessEntityID] 
                        JOIN [HumanResources].[Department] d ON edh.[DepartmentID] = d.[DepartmentID] 
                    WHERE edh.EndDate IS NULL
                ) managers
                on employees.OrganizationNode = managers.OrganizationNode
            group by employees.department, employees.JobTitle, employees.OrganizationLevel,
                managers.name, 
                managers.JobTitle, 
                employees.[name], employees.BusinessEntityID, employees.StartDate
            order by employees.OrganizationLevel,  employees.JobTitle";

        private const string singleitemSelect =
            @"
                select 
                    SalesOrderID, Quantity, UnitPrice, [Name]
                from
                (
                    SELECT 
                        SalesOrderID, 
                        max(productid) as productid,
                        max(OrderQty) as Quantity,
                        MAX(UnitPrice) as UnitPrice
                    FROM Sales.SalesOrderDetail
                    GROUP BY SalesOrderID
                    HAVING COUNT(SalesOrderID) = 1 and max(OrderQty) > 1
                ) sales
                join
                (
                    SELECT 
                        productid,
                        ProductModelID
                    FROM [Production].[Product]
                ) product
                on sales.productid = product.ProductID
                join
                (
                    SELECT 
                        ProductModelID,
                        [Name]
                    FROM [Production].[ProductModel]
                ) productmodel
                on product.ProductModelID = productmodel.ProductModelID
                order by SalesOrderID
            ";
        private const string connectionString = "Server=TCP:192.168.0.7;Database=AdventureWorks2019;Integrated Security=SSPI;Connect Timeout=30;";

        static void Main(string[] args)
        {
            using(new ExcelUILanguageHelper())
            {
                string fileName = Environment.GetFolderPath(Environment.SpecialFolder.Desktop) + "\\" + "ExcelReport-1.xlsx";

                Excel.Application xlsApp;
                Excel.Workbook xlsWorkbook;
                Excel.Worksheet xlsSingleItemOrder;
                Excel.Worksheet xlsManagers;
                object misValue = System.Reflection.Missing.Value;

                // Remove the old excel report file
                try
                {
                    FileInfo oldFile = new FileInfo(fileName);
                    if(oldFile.Exists)
                    {
                        File.SetAttributes(oldFile.FullName, FileAttributes.Normal);
                        oldFile.Delete();
                    }
                }
                catch(Exception ex)
                {
                    MessageBox.Show("Error removing old Excel report: " + ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Stop);
                    return;
                }

                try
                {
                    // In case that database cannot be open, don't open Excel app
                    using SqlConnection conn = new SqlConnection(connectionString);
                    conn.Open();

                    xlsApp = new Excel.Application();
                    xlsWorkbook = xlsApp.Workbooks.Add(misValue);
                    xlsWorkbook.Sheets.Add(); // Add a second sheet

                    // Create the header for the first sheet
                    xlsSingleItemOrder = (Excel.Worksheet)xlsWorkbook.Sheets[1];
                    xlsSingleItemOrder.Name = "Single Item Sales";
                    xlsSingleItemOrder.Cells[1, 1] = "Single Item Sales";
                    Excel.Range rangeSingleItems = xlsSingleItemOrder.get_Range("A1", "D1");
                    rangeSingleItems.Merge(1);
                    rangeSingleItems.Borders.Color = Color.Black.ToArgb();
                    rangeSingleItems.Interior.Color = Color.Yellow.ToArgb();
                    rangeSingleItems.Font.Name = "Courier New";
                    rangeSingleItems.Font.Size = 14;
                    rangeSingleItems.Cells.HorizontalAlignment = Excel.XlHAlign.xlHAlignCenterAcrossSelection;

                    // Create the header for the second sheet
                    xlsManagers = (Excel.Worksheet)xlsWorkbook.Sheets[2];
                    xlsManagers.Name = "Company Organization Structure";
                    xlsManagers.Cells[1, 1] = "Company Organization Structure";
                    Excel.Range rangeManagers = xlsManagers.get_Range("A1", "G1");
                    rangeManagers.Merge(1);
                    rangeManagers.Borders.Color = Color.Black.ToArgb();
                    rangeManagers.Interior.Color = Color.Yellow.ToArgb();
                    rangeManagers.Font.Name = "Courier New";
                    rangeManagers.Font.Size = 14;
                    rangeManagers.Cells.HorizontalAlignment = Excel.XlHAlign.xlHAlignCenterAcrossSelection;

                    // Process single item orders
                    using SqlCommand cmd = new SqlCommand(singleitemSelect, conn);
                    using SqlDataReader dr = cmd.ExecuteReader();

                    int i = 3;
                    if(dr.HasRows)
                    {
                        for(int j = 0; j < dr.FieldCount; ++j)
                        {
                            xlsSingleItemOrder.Cells[i, j + 1] = dr.GetName(j);
                        }
                        ++i;
                    }

                    while(dr.Read())
                    {
                        for(int j = 1; j <= dr.FieldCount; ++j)
                            xlsSingleItemOrder.Cells[i, j] = dr.GetValue(j - 1);
                        ++i;
                    }
                    dr.Close();
                    rangeSingleItems = xlsSingleItemOrder.get_Range("A2", "I" + (i + 2).ToString());
                    rangeSingleItems.Columns.AutoFit();

                    // Process managers
                    i = 3;

                    using SqlCommand cmdmanagers = new SqlCommand(sqlselectManagers, conn);
                    using SqlDataReader drmanager = cmdmanagers.ExecuteReader();
                    if(drmanager.HasRows)
                    {
                        for(int j = 0; j < drmanager.FieldCount; ++j)
                        {
                            xlsManagers.Cells[i, j + 1] = drmanager.GetName(j);
                        }
                        ++i;
                    }

                    while(drmanager.Read())
                    {
                        for(int j = 1; j <= drmanager.FieldCount; ++j)
                            xlsManagers.Cells[i, j] = drmanager.GetValue(j - 1);
                        ++i;
                    }
                    drmanager.Close();

                    rangeManagers = xlsManagers.get_Range("A2", "I" + (i + 2).ToString());
                    rangeManagers.Columns.AutoFit();

                    // Save the Excel file
                    xlsWorkbook.SaveAs(fileName, Excel.XlFileFormat.xlWorkbookDefault, misValue, misValue, misValue, misValue,
                        Excel.XlSaveAsAccessMode.xlExclusive, Excel.XlSaveConflictResolution.xlLocalSessionChanges, misValue, misValue, misValue, misValue);
                    xlsWorkbook.Close(true, misValue, misValue);
                    xlsApp.Quit();

                    ReleaseObject(xlsSingleItemOrder);
                    ReleaseObject(xlsWorkbook);
                    ReleaseObject(xlsApp);

                    if(MessageBox.Show("Excel report has been created on your desktop\nWould you like to open it?", "Created Excel report",
                        MessageBoxButtons.YesNo, MessageBoxIcon.Information, MessageBoxDefaultButton.Button2) == DialogResult.Yes)
                    {
                        Process.Start(fileName);
                    }
                }
                catch(Exception ex)
                {
                    MessageBox.Show("Error creating Excel report: " + ex.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Stop);
                }
            }
        }

        static private void ReleaseObject(object obj)
        {
            try
            {
                System.Runtime.InteropServices.Marshal.ReleaseComObject(obj);
                obj = null;
            }
            catch(Exception ex)
            {
                obj = null;
                MessageBox.Show("Exception occured while releasing object " + ex.ToString());
            }
            finally
            {
                GC.Collect();
            }
        }
    }
}
